from __future__ import absolute_import, unicode_literals

from sphinx.ext import autodoc as _autodoc
from sphinx.util import inspect


def wrapped_getargspec(fun, *args, **kwargs):
    while 1:
        try:
            wrapped = fun.__wrapped__
            if wrapped is fun:
                break
            fun = wrapped
        except AttributeError:
            break
    return inspect.getargspec(fun, *args, **kwargs)
_autodoc.getargspec = wrapped_getargspec


def setup(app):
    app.require_sphinx('1.0')



from __future__ import absolute_import, unicode_literals

import sys

__all__ = ['bytes_if_py2']

if sys.version_info[0] >= 3:

    def bytes_if_py2(s):
        return s
else:

    def bytes_if_py2(s):  # noqa
        if isinstance(s, unicode):
            return s.encode()
        return s



from __future__ import absolute_import, unicode_literals

import os
import pickle
import re

from sphinx.builders import Builder

ERR_INVALID_REGEX = 'Invalid regex {0!r} in apicheck_ignore_modules: {1!r}'


class BaseBuilder(Builder):

    def get_outdated_docs(self):
        return '{0} overview'.format(self.name)

    def finish(self):
        picklepath = os.path.join(self.outdir, self.pickle_filename)
        with open(picklepath, mode='wb') as fh:
            pickle.dump(self.as_dict(), fh)

    def compile_regex(self, regex):
        if not regex.startswith('^'):
            regex = '^{0}'.format(regex)
        if not regex.endswith('$'):
            regex = '{0}$'.format(regex)
        try:
            return re.compile(regex)
        except Exception as exc:
            self.warn(ERR_INVALID_REGEX.format(regex, exc))

    def compile_regexes(self, regexes):
        return [self.compile_regex(regex) for regex in regexes]



from __future__ import absolute_import, unicode_literals

from .utils import bytes_if_py2


def setup(app):
    app.add_crossref_type(
        directivename=bytes_if_py2('setting'),
        rolename=bytes_if_py2('setting'),
        indextemplate=bytes_if_py2('pair: %s; setting'),
    )



"""

Sphinx Configuration Reference Checker
======================================

This builder extension makes sure all settings in the documented
package are represented in the configuration reference (
meaning they all have ``.. setting::`` directives).

Usage
-----

.. code-block:: console

    $ sphinx-build -b configcheck -d _build/doctrees . _build/configcheck

Configuration
-------------

configcheck_ignore_settings
~~~~~~~~~~~~~~~~~~~~~~~~~~~

List of settings to ignore, either as setting names or regexes.

Example:

.. code-block:: python

    configcheck_ignore_settings = [
        'USE_TZ',
        r'.*SECRET.*',
    ]

configcheck_project_settings
~~~~~~~~~~~~~~~~~~~~~~~~~~~~

A function returning a set of all setting names.

Example:

.. code-block:: python

    def configcheck_project_settings():
        from django import conf

        return set(conf._all_settings)


configcheck_should_ignore
~~~~~~~~~~~~~~~~~~~~~~~~~

Optional function that can be used in addition to
``configcheck_ignore_settings`` to ignore setting names programmatically.


Example:

.. code-block:: python

    def configcheck_should_ignore(setting):
        from django import conf
        return conf.is_deprecated(setting)


"""
from __future__ import absolute_import, unicode_literals

from six import iterkeys as keys

from sphinx.util.console import bold, green, red

from .builders import BaseBuilder
from .utils import bytes_if_py2

ERR = 'ERROR'
ERR_MISSING_DOC = '{error}: Setting not documented: {name}'
OK_STATUS = 'OK: All settings documented :o)'


class ConfigCheckBuilder(BaseBuilder):

    name = 'configcheck'
    pickle_filename = 'configcheck.pickle'

    def init(self):
        self.ignore_patterns = self.compile_regexes(
            self.config.configcheck_ignore_settings,
        )
        self.should_ignore = (
            self.config.configcheck_should_ignore or (lambda s: False))
        self.project_settings = self.config.configcheck_project_settings
        self.undocumented = set()

    def is_ignored_setting(self, setting):
        return self.should_ignore(setting) or any(
            regex.match(setting) for regex in self.ignore_patterns)

    def write(self, *ignored):
        self.check_missing()

    def documented_settings(self):
        return {
            name for reftype, name in keys(
                self.app.env.domaindata['std']['objects'])
            if reftype == 'setting'
        }

    def check_missing(self):
        all_settings = self.project_settings()
        documented_settings = self.documented_settings()
        self.undocumented.update(
            setting for setting in all_settings ^ documented_settings
            if not self.is_ignored_setting(setting)
        )

        for setting in self.undocumented:
            self.app.statuscode = 2
            print(ERR_MISSING_DOC.format(
                error=red(ERR),
                name=bold(setting),
            ))
        if not self.app.statuscode:
            print(green(OK_STATUS))

    def as_dict(self):
        return {
            'undocumented': self.undocumented,
        }


def setup(app):
    app.add_builder(ConfigCheckBuilder)
    app.add_config_value(
        bytes_if_py2('configcheck_ignore_settings'), [], False)
    app.add_config_value(
        bytes_if_py2('configcheck_project_settings'), None, False)
    app.add_config_value(
        bytes_if_py2('configcheck_should_ignore'), None, False)



"""Stolen from sphinxcontrib-issuetracker.

Had to modify this as the original will make one Github API request
per issue, which is not at all needed if we just want to link to issues.

"""
from __future__ import absolute_import, unicode_literals

import re
import sys

from collections import namedtuple

from docutils import nodes
from docutils.transforms import Transform
from sphinx.roles import XRefRole
from sphinx.addnodes import pending_xref

URL = 'https://github.com/{project}/issues/{issue_id}'

Issue = namedtuple('Issue', ('id', 'title', 'url'))

if sys.version_info[0] == 3:
    str_t = text_t = str
else:
    str_t = basestring
    text_t = unicode


class IssueRole(XRefRole):
    innernodeclass = nodes.inline


class Issues(Transform):
    default_priority = 999

    def apply(self):
        config = self.document.settings.env.config
        github_project = config.github_project
        issue_pattern = config.github_issue_pattern
        if isinstance(issue_pattern, str_t):
            issue_pattern = re.compile(issue_pattern)
        for node in self.document.traverse(nodes.Text):
            parent = node.parent
            if isinstance(parent, (nodes.literal, nodes.FixedTextElement)):
                continue
            text = text_t(node)
            new_nodes = []
            last_issue_ref_end = 0
            for match in issue_pattern.finditer(text):
                head = text[last_issue_ref_end:match.start()]
                if head:
                    new_nodes.append(nodes.Text(head))
                last_issue_ref_end = match.end()
                issuetext = match.group(0)
                issue_id = match.group(1)
                refnode = pending_xref()
                refnode['reftarget'] = issue_id
                refnode['reftype'] = 'issue'
                refnode['github_project'] = github_project
                reftitle = issuetext
                refnode.append(nodes.inline(
                    issuetext, reftitle, classes=['xref', 'issue']))
                new_nodes.append(refnode)
            if not new_nodes:
                continue
            tail = text[last_issue_ref_end:]
            if tail:
                new_nodes.append(nodes.Text(tail))
            parent.replace(node, new_nodes)


def make_issue_reference(issue, content_node):
    reference = nodes.reference()
    reference['refuri'] = issue.url
    if issue.title:
        reference['reftitle'] = issue.title
    reference.append(content_node)
    return reference


def resolve_issue_reference(app, env, node, contnode):
    if node['reftype'] != 'issue':
        return
    issue_id = node['reftarget']
    project = node['github_project']

    issue = Issue(issue_id, None, URL.format(project=project,
                                             issue_id=issue_id))
    conttext = text_t(contnode[0])
    formatted_conttext = nodes.Text(conttext.format(issue=issue))
    formatted_contnode = nodes.inline(conttext, formatted_conttext,
                                      classes=contnode['classes'])
    return make_issue_reference(issue, formatted_contnode)


def init_transformer(app):
    app.add_transform(Issues)


def setup(app):
    app.require_sphinx('1.0')
    app.add_role('issue', IssueRole())

    app.add_config_value('github_project', None, 'env')
    app.add_config_value('github_issue_pattern',
                         re.compile(r'[Ii]ssue #(\d+)'), 'env')

    app.connect(str('builder-inited'), init_transformer)
    app.connect(str('missing-reference'), resolve_issue_reference)



"""Sphinx Celery Theme."""
from __future__ import absolute_import, unicode_literals

import os
import re

from collections import namedtuple

__version__ = '1.3.1'
__author__ = 'Ask Solem'
__contact__ = 'ask@celeryproject.org'
__homepage__ = 'http://github.com/celery/sphinx_celery'
__docformat__ = 'restructuredtext'

# -eof meta-

__all__ = ['get_html_templates_path', 'get_html_theme_path']

version_info_t = namedtuple('version_info_t', (
    'major', 'minor', 'micro', 'releaselevel', 'serial',
))

# bumpversion can only search for {current_version}
# so we have to parse the version here.
_temp = re.match(
    r'(\d+)\.(\d+).(\d+)(.+)?', __version__).groups()
VERSION = version_info = version_info_t(
    int(_temp[0]), int(_temp[1]), int(_temp[2]), _temp[3] or '', '')
del(_temp)
del(re)


def get_html_theme_path():
    return os.path.abspath(os.path.dirname(os.path.dirname(__file__)))


def get_html_templates_path():
    return os.path.join(
        os.path.abspath(os.path.dirname((__file__))),
        'templates',
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

import os
import re
import sys
import codecs

NAME = 'sphinx_celery'
extra = {}

# -*- Classifiers -*-

classes = """
    Development Status :: 5 - Production/Stable
    Programming Language :: Python
    Programming Language :: Python :: 2
    Programming Language :: Python :: 2.7
    Programming Language :: Python :: 2.6
    Programming Language :: Python :: 3
    Programming Language :: Python :: 3.3
    Programming Language :: Python :: 3.4
    Programming Language :: Python :: 3.5
    License :: OSI Approved :: BSD License
    Intended Audience :: Developers
    Operating System :: OS Independent
"""
classifiers = [s.strip() for s in classes.split('\n') if s]

# -*- Distribution Meta -*-

re_meta = re.compile(r'__(\w+?)__\s*=\s*(.*)')
re_doc = re.compile(r'^"""(.+?)"""')


def add_default(m):
    attr_name, attr_value = m.groups()
    return ((attr_name, attr_value.strip("\"'")),)


def add_doc(m):
    return (('doc', m.groups()[0]),)

pats = {re_meta: add_default, re_doc: add_doc}
here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'sphinx_celery', '__init__.py')) as meta_fh:
    meta = {}
    for line in meta_fh:
        if line.strip() == '# -eof meta-':
            break
        for pattern, handler in pats.items():
            m = pattern.match(line.strip())
            if m:
                meta.update(handler(m))

# -*- Installation Requires -*-

py_version = sys.version_info
is_jython = sys.platform.startswith('java')
is_pypy = hasattr(sys, 'pypy_version_info')


def strip_comments(l):
    return l.split('#', 1)[0].strip()


def _pip_requirement(req):
    if req.startswith('-r '):
        _, path = req.split()
        return reqs(*path.split('/'))
    return [req]


def _reqs(*f):
    return [
        _pip_requirement(r) for r in (
            strip_comments(l) for l in open(
                os.path.join(os.getcwd(), 'requirements', *f)).readlines()
        ) if r]


def reqs(*f):
    return [req for subreq in _reqs(*f) for req in subreq]

# -*- Long Description -*-

if os.path.exists('README.rst'):
    long_description = codecs.open('README.rst', 'r', 'utf-8').read()
else:
    long_description = 'See http://pypi.python.org/pypi/sphinx_celery/'

# -*- Entry Points -*- #

# -*- %%% -*-


setup(
    name=NAME,
    version=meta['version'],
    description=meta['doc'],
    author=meta['author'],
    author_email=meta['contact'],
    url=meta['homepage'],
    platforms=['any'],
    license='BSD',
    packages=find_packages(exclude=['ez_setup', 'tests', 'tests.*']),
    package_data={
        'sphinx_celery': [
            'theme.conf',
        ],
        os.path.join('sphinx_celery', 'static'): [
            'celery.css_t',
        ],
        os.path.join('sphinx_celery', 'templates'): [
            'page.html',
        ],
    },
    include_package_data=True,
    zip_safe=False,
    install_requires=reqs('default.txt'),
    tests_require=reqs('test.txt'),
    test_suite='nose.collector',
    classifiers=classifiers,
    entry_points={
        'sphinx_themes': [
            'path = sphinx_celery:get_html_theme_path',
        ],
    },
    long_description=long_description,
    **extra)



"""

Sphinx Autodoc coverage checker.
================================

This builder extension makes sure all modules in the documented
package is represented in the autodoc API reference.

Usage
-----

.. code-block:: console

    $ sphinx-build -b apicheck -d _build/doctrees . _build/apicheck

Configuration
-------------

apicheck_ignore_modules
~~~~~~~~~~~~~~~~~~~~~~~

List of modules to ignore, either as module names or regexes.

Example:

.. code-block:: python

    apicheck_ignore_modules = [
        'django.utils.functional',
        r'django.db.*',
    ]

Test packages are ignored by default, even if this setting is defined.

apicheck_package
~~~~~~~~~~~~~~~~

The package to verify, can be the fully-qualified name of a module
or an actual module.

Example:

.. code-block:: python

    apicheck_package = 'django'

Default is the value of the ``project`` configuration key in all lowercase.


apicheck_domains
~~~~~~~~~~~~~~~~

List of domains to check.

Default is ``['py']`` and Python is the only domain currently supported.

"""
from __future__ import absolute_import, unicode_literals

import importlib
import os

from collections import defaultdict
from six import string_types

from sphinx.ext import autodoc
from sphinx.util.console import bold, darkgreen, green, red

from .builders import BaseBuilder
from .utils import bytes_if_py2

DEFAULT_IGNORE = [r'.*?\.tests.*']

TITLEHEADER = '='
SUBHEADER = '-'

ERR = 'ERROR'
ERR_MISSING = '{error}: In index but module does not exist: {module}'
ERR_UNDOCUMENTED = 'Undocumented Autodoc Modules'
OK_STATUS = 'OK: All modules documented :o)'

NOK_STATUS = """
{title}

{undocumented}\
"""

DOMAIN_FORMAT = """\
{domain}

{modules}
"""

MODULE_FORMAT = '- {module}'


class ModuleDocumenter(autodoc.ModuleDocumenter):
    missing_modules = set()

    def import_object(self):
        if not super(ModuleDocumenter, self).import_object():
            self.missing_modules.add(self.modname)
            return False
        return True


def title(s, spacing=2, sep=TITLEHEADER):
    return '\n'.join([
        sep * (len(s) + spacing),
        '{0}{1}{0}'.format(' ' * (spacing // 2), red(s)),
        sep * (len(s) + spacing),
    ])


def header(s, sep=SUBHEADER):
    return '\n'.join([bold(s), sep * len(s)])


def find_python_modules(package):
    if isinstance(package, string_types):
        package = importlib.import_module(package)
    name, path = package.__name__, package.__file__
    current_dist_depth = len(name.split('.')) - 1
    current_dist = os.path.join(os.path.dirname(path),
                                *([os.pardir] * current_dist_depth))
    abs = os.path.abspath(current_dist)
    dist_name = os.path.basename(abs)

    for dirpath, dirnames, filenames in os.walk(abs):
        package = (dist_name + dirpath[len(abs):]).replace('/', '.')
        if '__init__.py' in filenames:
            yield package
            for filename in filenames:
                if filename.endswith('.py') and filename != '__init__.py':
                    yield '.'.join([package, filename])[:-3]


class APICheckBuilder(BaseBuilder):

    name = 'apicheck'
    pickle_filename = 'apicheck.pickle'

    find_modules = {
        'py': find_python_modules,
    }

    def init(self):
        self.ignore_patterns = self.compile_regexes(
            self.config.apicheck_ignore_modules + DEFAULT_IGNORE,
        )
        self.check_domains = self.config.apicheck_domains
        self.check_package = (
            self.config.apicheck_package or self.config.project.lower())

        self.undocumented = defaultdict(list)
        self.all_modules = defaultdict(set)

    def is_ignored_module(self, module):
        return any(regex.match(module) for regex in self.ignore_patterns)

    def write(self, *ignored):
        for domain in self.check_domains:
            self.build_coverage(domain)
        self.check_missing()
        if not self.app.statuscode:
            self.write_coverage(self.check_domains)

    def build_coverage(self, domain):
        self.all_modules[domain].update(self.find_modules[domain](
            self.check_package,
        ))
        self.undocumented[domain].extend(self.find_undocumented(
            domain, self.env.domaindata[domain]['modules'],
        ))

    def find_undocumented(self, domain, documented):
        return (
            mod for mod in self.all_modules[domain]
            if mod not in documented and not self.is_ignored_module(mod)
        )

    def write_coverage(self, domains):
        status = any(self.undocumented.values())
        if status:
            self.app.statuscode = 2
            print(self.format_undocumented_domains(domains))
        else:
            print(green(OK_STATUS))

    def check_missing(self):
        for mod in ModuleDocumenter.missing_modules:
            self.app.statuscode = 3
            print(ERR_MISSING.format(
                error=red(ERR),
                module=bold(mod),
            ))

    def format_undocumented_domains(self, domains):
        return NOK_STATUS.format(
            title=title(ERR_UNDOCUMENTED),
            undocumented='\n'.join(
                self.format_undocumented_domain(domain) for domain in domains
            ),
        )

    def format_undocumented_domain(self, domain):
        return DOMAIN_FORMAT.format(domain=header(domain), modules='\n'.join(
            self.format_undocumented_module(module)
            for module in self.undocumented[domain]
        ))

    def format_undocumented_module(self, module):
        return MODULE_FORMAT.format(module=darkgreen(module))

    def as_dict(self):
        return {
            'undocumented': dict(self.undocumented),
        }


def setup(app):
    app.add_builder(APICheckBuilder)
    app.add_config_value(
        bytes_if_py2('apicheck_ignore_modules'), [], False)
    app.add_config_value(
        bytes_if_py2('apicheck_domains'), ['py'], False)
    app.add_config_value(
        bytes_if_py2('apicheck_package'), None, False)
    reg = autodoc.AutoDirective._registry
    reg[ModuleDocumenter.objtype] = ModuleDocumenter



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import os
import sys

from . import get_html_templates_path

PY3 = sys.version_info[0] >= 3

LINKCODE_URL = 'https://github.com/{proj}/tree/{branch}/{filename}.py'
GITHUB_BRANCH = 'master'

EXTENSIONS = [
    'sphinx.ext.autodoc',
    'sphinx.ext.intersphinx',
    'sphinx.ext.todo',
    'sphinx.ext.coverage',
    'sphinx.ext.imgmath',
    'sphinx.ext.ifconfig',
    'sphinx.ext.viewcode',
    'sphinx.ext.extlinks',

    'sphinx_celery.autodocargspec',
    'sphinx_celery.github_issues',
    'sphinx_celery.signal_crossref',
    'sphinx_celery.setting_crossref',
    'sphinx_celery.apicheck',
    'sphinx_celery.configcheck',
]

INTERSPHINX_MAPPING = {
    'python': ('http://docs.python.org/dev/', None),
    'sphinx': ('http://www.sphinx-doc.org/en/stable/', None),
    'kombu': ('http://kombu.readthedocs.io/en/master/', None),
    'celery': ('http://docs.celeryproject.org/en/master', None),
    'djcelery': ('http://django-celery.readthedocs.io/en/latest/', None),
    'cyme': ('http://cyme.readthedocs.io/en/latest/', None),
    'amqp': ('http://amqp.readthedocs.io/en/latest/', None),
    'vine': ('http://vine.readthedocs.io/en/latest/', None),
    'flower': ('http://flower.readthedocs.io/en/latest/', None),
    'redis': ('http://redis-py.readthedocs.io/en/latest/', None),
    'django': ('http://django.readthedocs.io/en/latest/', None),
    'boto': ('http://boto.readthedocs.io/en/latest/', None),
    'sqlalchemy': ('http://sqlalchemy.readthedocs.io/en/latest', None),
    'kazoo': ('http://kazoo.readthedocs.io/en/latest/', None),
    'msgpack': ('http://pythonhosted.org/msgpack-python/', None),
    'riak': ('http://basho.github.io/riak-python-client/', None),
    'pylibmc': ('http://sendapatch.se/projects/pylibmc/', None),
    'eventlet': ('http://eventlet.net/doc/', None),
    'gevent': ('http://gevent.org/', None),
    'pyOpenSSL': ('http://pyopenssl.readthedocs.io/en/stable/', None),
    'nose': ('http://nose.readthedocs.io/en/latest', None),
    'tox': ('http://tox.readthedocs.io/en/latest', None),
}

string_types = (str,) if PY3 else (basestring,)


def add_paths(config_file, path_additions):
    this = os.path.dirname(os.path.abspath(config_file))

    sys.path.insert(0, os.path.join(this, os.pardir))
    for path in path_additions:
        sys.path.append(os.path.join(this, path))


def configure_django(django_settings, **config):
    if django_settings:
        os.environ['DJANGO_SETTINGS_MODULE'] = django_settings
    else:
        from django.conf import settings  # noqa
        if not settings.configured:
            settings.configure(**config)
    try:
        from django import setup as django_setup
    except ImportError:
        pass
    else:
        django_setup()


def import_package(package):
    if isinstance(package, string_types):
        return __import__(package)
    return package


def prepare_intersphinx_mapping(project, mapping,
                                include, exclude, **extra):
    if include:
        mapping = dict((k, v) for k, v in mapping.items() if k in include)
    if exclude:
        mapping = dict((k, v) for k, v in mapping.items() if k not in exclude)
    mapping = dict(mapping, **extra)

    # Remove project itself from intersphinx
    mapping.pop(project.lower(), None)

    return mapping


def create_linkcode_resolver(linkcode_url, github_project, github_branch):
    def linkcode_resolve(domain, info):
        if domain != 'py' or not info['module']:
            return
        filename = info['module'].replace('.', '/')
        return linkcode_url.format(
            proj=github_project,
            branch=github_branch,
            filename=filename,
        )
    return linkcode_resolve


def build_config(
        package, config_file, project,
        author=None,
        author_name=None,
        github_project=None,
        webdomain=None,
        canonical_url=None,
        canonical_stable_url=None,
        canonical_dev_url=None,
        django_settings=None,
        configure_django_settings={},
        copyright=None,
        publisher=None,
        description='',
        path_additions=['_ext'],
        version_dev=None,
        version_stable=None,
        extensions=EXTENSIONS,
        extra_extensions=[],
        linkcode_url=LINKCODE_URL,
        github_branch=GITHUB_BRANCH,
        master_doc='index',
        html_logo=None,
        html_prepend_sidebars=[],
        templates_path=None,
        latex_logo=None,
        intersphinx_mapping=INTERSPHINX_MAPPING,
        extra_intersphinx_mapping={},
        include_intersphinx=frozenset(),
        exclude_intersphinx=frozenset(),
        spelling_lang='en_US',
        spelling_show_suggestions=True,
        extlinks=None,
        **kwargs):
    add_paths(config_file, path_additions)
    if configure_django_settings or django_settings:
        configure_django(django_settings, **configure_django_settings or {})
    package = import_package(package)
    description = description or package.__doc__
    author = author or package.__author__
    author_name = author_name or author
    extlinks = extlinks or {}

    extlinks.setdefault('sha', (
        'https://github.com/{0}/commit/%s'.format(github_project),
        'GitHub SHA@',
    ))
    extlinks.setdefault('github_branch', (
        'https://github.com/{0}/tree/%s'.format(github_project),
        'GitHub branch',
    ))
    extlinks.setdefault('github_user', (
        'https://github.com/%s/', '@',
    ))
    extlinks.setdefault('pypi', (
        'https://pypi.python.org/pypi/%s/', '',
    ))

    if not canonical_dev_url:
        canonical_dev_url = '/'.join([
            canonical_url.rstrip('/'), 'en', 'master',
        ])
    if not canonical_stable_url:
        canonical_stable_url = '/'.join([
            canonical_url.rstrip('/'), 'en', 'latest',
        ])

    if templates_path is None:
        templates_path = ['_templates']
    if version_dev:
        templates_path.append(get_html_templates_path())

    version = '.'.join(map(str, package.VERSION[0:2]))

    extensions = extensions + extra_extensions
    if os.environ.get('SPELLCHECK'):
        extensions.append('sphinxcontrib.spelling')

    conf = dict(
        extensions=extensions + extra_extensions,

        project=project,
        github_project=github_project,

        html_show_sphinx=False,

        # Add any paths that contain templates here,
        #   relative to this directory.
        templates_path=templates_path,

        # The suffix of source filenames.
        source_suffix='.rst',

        # The master toctree document.
        master_doc=master_doc,

        copyright='{0}, {1}'.format(copyright, author),

        # The short X.Y version.
        version=version,

        # The full version, including alpha/beta/rc tags.
        release=package.__version__,

        exclude_patterns=['_build', 'Thumbs.db', '.DS_Store'],

        # If true, '()' will be appended to :func: etc. cross-reference text.
        add_function_parentheses=True,

        linkcode_resolve=create_linkcode_resolver(
            linkcode_url, github_project, github_branch,
        ),

        intersphinx_mapping=prepare_intersphinx_mapping(
            project,
            intersphinx_mapping,
            include_intersphinx,
            exclude_intersphinx,
            **extra_intersphinx_mapping
        ),

        # The name of the Pygments (syntax highlighting) style to use.
        pygments_style='colorful',

        # Add any paths that contain custom static files
        # (such as style sheets) here, relative to this directory.
        # They are copied after the builtin static files, so a file named
        # "default.css" will overwrite the builtin "default.css".
        html_static_path=['_static'],

        html_use_smartypants=True,

        add_module_names=True,
        highlight_language='python3',

        # If true, `todo` and `todoList` produce output,
        # else they produce nothing.
        todo_include_todos=True,

        # If false, no module index is generated.
        html_use_modindex=True,

        # If false, no index is generated.
        html_use_index=True,

        html_logo=html_logo,

        html_context={
            'version_dev': version_dev or version,
            'version_stable': version_stable or version,
            'canonical_stable_url': canonical_stable_url,
            'canonical_dev_url': canonical_dev_url,
        },

        man_pages=[
            (master_doc, project.lower(),
             u'{0} Documentation'.format(project), [author_name], 1)
        ],

        # Grouping the document tree into Texinfo files. List of tuples
        # (source start file, target name, title, author,
        #  dir menu entry, description, category)
        texinfo_documents=[
            (master_doc, project, u'{0} Documentation'.format(project),
             author_name, project, description,
             'Miscellaneous'),
        ],
        latex_logo=latex_logo or html_logo,

        latex_documents=[
            ('index', '{0}.tex'.format(project),
             '{0} Documentation'.format(project), author, 'manual'),
        ],
        html_theme='sphinx_celery',
        html_sidebars={
            'index': list(html_prepend_sidebars) + [
                'sourcelink.html',
                'searchbox.html',
            ],
            '**': list(html_prepend_sidebars) + [
                'relations.html',
                'sourcelink.html',
                'searchbox.html',
            ],
        },
        # Bibliographic Dublin Core info.
        epub_title='Celery Manual, Version {0}'.format(version),
        epub_author=author_name,
        epub_publisher=publisher or author_name,
        epub_copyright=copyright,

        # The language of the text. It defaults to the language option
        # or en if the language is not set.
        epub_language='en',

        # The scheme of the identifier. Typical schemes are ISBN or URL.
        epub_scheme='ISBN',

        # The unique identifier of the text. This can be a ISBN number
        # or the project homepage.
        epub_identifier=webdomain,

        # A unique identification for the text.
        epub_uid='{0} Manual, Version {0}'.format(project, version),

        # A list of files that should not be packed into the epub file.
        epub_exclude_files=['search.html'],

        # The depth of the table of contents in toc.ncx.
        epub_tocdepth=3,

        # -- spelling
        spelling_lang=spelling_lang,
        spelling_show_suggestions=spelling_show_suggestions,

        # -- extlinks
        extlinks=extlinks,
    )
    return dict(conf, **kwargs)
